package javax.management;

import java.security.AccessController;
import com.sun.jmx.mbeanserver.GetPropertyAction;
import sun.reflect.misc.ReflectUtil;
import java.util.logging.Level;
import com.sun.jmx.defaults.JmxProperties;
import javax.management.loading.ClassLoaderRepository;
import java.util.Iterator;
import java.util.ArrayList;

public class MBeanServerFactory
{
    private static MBeanServerBuilder builder;
    private static final ArrayList<MBeanServer> mBeanServerList;
    
    private MBeanServerFactory() {
    }
    
    public static void releaseMBeanServer(MBeanServer mBeanServer) {
        checkPermission("releaseMBeanServer");
        removeMBeanServer(mBeanServer);
    }
    
    public static MBeanServer createMBeanServer() {
        return createMBeanServer(null);
    }
    
    public static MBeanServer createMBeanServer(String s) {
        checkPermission("createMBeanServer");
        MBeanServer mBeanServer = newMBeanServer(s);
        addMBeanServer(mBeanServer);
        return mBeanServer;
    }
    
    public static MBeanServer newMBeanServer() {
        return newMBeanServer(null);
    }
    
    public static MBeanServer newMBeanServer(String s) {
        checkPermission("newMBeanServer");
        MBeanServerBuilder newMBeanServerBuilder = getNewMBeanServerBuilder();
        synchronized (newMBeanServerBuilder) {
            MBeanServerDelegate mBeanServerDelegate = newMBeanServerBuilder.newMBeanServerDelegate();
            if (mBeanServerDelegate == null) {
                throw new JMRuntimeException("MBeanServerBuilder.newMBeanServerDelegate() returned null");
            }
            MBeanServer mBeanServer = newMBeanServerBuilder.newMBeanServer(s, null, mBeanServerDelegate);
            if (mBeanServer == null) {
                throw new JMRuntimeException("MBeanServerBuilder.newMBeanServer() returned null");
            }
            return mBeanServer;
        }
    }
    
    public static synchronized ArrayList<MBeanServer> findMBeanServer(String s) {
        checkPermission("findMBeanServer");
        if (s == null) {
            return new ArrayList(MBeanServerFactory.mBeanServerList);
        }
        ArrayList<MBeanServer> list = new ArrayList();
        for (MBeanServer e : MBeanServerFactory.mBeanServerList) {
            if (s.equals(mBeanServerId(e))) {
                list.add(e);
            }
            else {
                continue;
            }
        }
        return list;
    }
    
    public static ClassLoaderRepository getClassLoaderRepository(MBeanServer mBeanServer) {
        return mBeanServer.getClassLoaderRepository();
    }
    
    private static String mBeanServerId(MBeanServer mBeanServer) {
        try {
            return (String)mBeanServer.getAttribute(MBeanServerDelegate.DELEGATE_NAME, "MBeanServerId");
        }
        catch (JMException obj) {
            JmxProperties.MISC_LOGGER.finest("Ignoring exception while getting MBeanServerId: " + obj);
            return null;
        }
    }
    
    private static void checkPermission(String s) throws SecurityException {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new MBeanServerPermission(s));
        }
    }
    
    private static synchronized void addMBeanServer(MBeanServer e) {
        MBeanServerFactory.mBeanServerList.add(e);
    }
    
    private static synchronized void removeMBeanServer(MBeanServer o) {
        if (!MBeanServerFactory.mBeanServerList.remove(o)) {
            JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINER, MBeanServerFactory.class.getName(), "removeMBeanServer(MBeanServer)", "MBeanServer was not in list!");
            throw new IllegalArgumentException("MBeanServer was not in list!");
        }
        return;
    }
    
    private static Class<?> loadBuilderClass(String s) throws ClassNotFoundException {
        ClassLoader contextClassLoader = Thread.currentThread().getContextClassLoader();
        if (contextClassLoader != null) {
            return contextClassLoader.loadClass(s);
        }
        return ReflectUtil.forName(s);
    }
    
    private static MBeanServerBuilder newBuilder(Class<?> obj) {
        try {
            return (MBeanServerBuilder)obj.newInstance();
        }
        catch (RuntimeException ex) {
            throw ex;
        }
        catch (Exception obj2) {
            throw new JMRuntimeException("Failed to instantiate a MBeanServerBuilder from " + obj + ": " + obj2, obj2);
        }
    }
    
    private static synchronized void checkMBeanServerBuilder() {
        try {
            String str = (String)AccessController.doPrivileged(new GetPropertyAction("javax.management.builder.initial"));
            try {
                Class<Object> loadBuilderClass;
                if (str == null || str.length() == 0) {
                    loadBuilderClass = MBeanServerBuilder.class;
                }
                else {
                    loadBuilderClass = loadBuilderClass(str);
                }
                if (MBeanServerFactory.builder != null && loadBuilderClass == MBeanServerFactory.builder.getClass()) {
                    return;
                }
                MBeanServerFactory.builder = newBuilder(loadBuilderClass);
            }
            catch (ClassNotFoundException obj) {
                throw new JMRuntimeException("Failed to load MBeanServerBuilder class " + str + ": " + obj, obj);
            }
        }
        catch (RuntimeException obj2) {
            if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINEST)) {
                JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINEST, MBeanServerFactory.class.getName(), "checkMBeanServerBuilder", "Failed to instantiate MBeanServerBuilder: " + obj2 + "\n\t\tCheck the value of the " + "javax.management.builder.initial" + " property.");
            }
            throw obj2;
        }
    }
    
    private static synchronized MBeanServerBuilder getNewMBeanServerBuilder() {
        checkMBeanServerBuilder();
        return MBeanServerFactory.builder;
    }
    
    static {
        MBeanServerFactory.builder = null;
        mBeanServerList = new ArrayList();
    }
}
